// 3dSprite.cpp
//
// Copyright (c) Nigel Thompson 1996
//
// Implementation for:
// C3dSprite
//

#include "stdafx.h"
#include "3dplus.h"

////////////////////////////////////////////////////////////////////////////
// C3dSprite

IMPLEMENT_DYNAMIC(C3dSprite, C3dFrame)


C3dSprite::C3dSprite()
{
}

// virtual 
C3dSprite::~C3dSprite()
{
}

BOOL C3dSprite::Create(C3dScene* pScene,
						double x, double y, double z,
						double scale,
						UINT uiIDBitmap,
						int iPhases)
{
	ASSERT(pScene);
	ASSERT(iPhases > 0);
	ASSERT(uiIDBitmap);

	// Create frame	and add it to scene list
	C3dFrame::Create(pScene);
	pScene->m_ShapeList.Append(this);

	// Put frame in requested position
	SetPosition(x, y, z);

	// Load texture map	image
	if (!m_Tex.C3dImage::Load(uiIDBitmap)) {
		TRACE("Failed to load texture image\n");
		return FALSE;
	}

	// Set number of phases
	m_Tex.C3dImage::SetNumPhases(iPhases);

	// Create texture map from image
	m_Tex.Create();

	// Set properties
	IDirect3DRMTexture* pITex = m_Tex.GetInterface();
	ASSERT(pITex);
	
	// Enable depth-scaling
	m_hr = pITex->SetDecalScale(TRUE);
	ASSERT(SUCCEEDED(m_hr));

	// Set initial size
	double a = (double)m_Tex.GetWidth()
			 / (double)m_Tex.GetHeight();
	m_hr = pITex->SetDecalSize(scale * a, scale);
	ASSERT(SUCCEEDED(m_hr));

	// Turn on transparency
	m_hr = pITex->SetDecalTransparency(TRUE);
	ASSERT(SUCCEEDED(m_hr));

	// Set the transparency color to black
	m_hr = pITex->SetDecalTransparentColor(RGB_MAKE(0, 0, 0));
 	ASSERT(SUCCEEDED(m_hr));

	// Set center bottom of image to map
	// to frame origin
	m_hr = pITex->SetDecalOrigin(m_Tex.GetWidth()/2,
								 m_Tex.GetHeight()-1);
	ASSERT(SUCCEEDED(m_hr));

	// Add texture as visual
	return AddVisual(&m_Tex);
}

BOOL C3dSprite::SetPhase(int iPhase)
{
	return m_Tex.SetPhase(iPhase);
}

