// texpage.cpp : implementation file
//

#include "stdafx.h"
#include "..\Include\3dPlus.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTexPage property page

IMPLEMENT_DYNCREATE(CTexPage, CClrBasePage)

CTexPage::CTexPage() : CClrBasePage(CTexPage::IDD)
{
	//{{AFX_DATA_INIT(CTexPage)
	m_iFace = 0;
	//}}AFX_DATA_INIT
}

CTexPage::~CTexPage()
{
}

void CTexPage::DoDataExchange(CDataExchange* pDX)
{
	CClrBasePage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTexPage)
	DDX_Control(pDX, _3D_IDC_IMAGE, m_cbImage);
	DDX_Control(pDX, _3D_IDC_FACECOUNT, m_wndFaceCount);
	DDX_Text(pDX, _3D_IDC_FACE, m_iFace);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTexPage, CClrBasePage)
	//{{AFX_MSG_MAP(CTexPage)
	ON_BN_CLICKED(_3D_IDC_BROWSE, OnBrowse)
	ON_BN_CLICKED(_3D_IDC_APPLY, OnApplyTexture)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTexPage message handlers

BOOL CTexPage::OnSetActive() 
{
	CheckRadioButton(_3D_IDC_RADIO1, _3D_IDC_RADIO2, _3D_IDC_RADIO1);
	CheckRadioButton(_3D_IDC_RADIO3, _3D_IDC_RADIO6, _3D_IDC_RADIO3);

	// fill the combo box with the list of available images
	ASSERT(m_pImgList);
	m_cbImage.ResetContent();
	POSITION pos = m_pImgList->GetHeadPosition();
	while (pos) {
		C3dImage* pImg = m_pImgList->GetNext(pos);
		ASSERT(pImg);
		m_cbImage.AddString(pImg->GetName());
	}
	m_cbImage.SetCurSel(0);

	ASSERT(m_pDlg);
	C3dShape* pShape = m_pDlg->GetShape();
	if (pShape) {
		int iFaces = pShape->GetFaceCount();
		char buf[64];
		sprintf(buf, "0 - %d", iFaces - 1);
		m_wndFaceCount.SetWindowText(buf);
	}

	return TRUE;
}

void CTexPage::OnBrowse() 
{
	// Show a file open dialog
	CFileDialog dlg(TRUE,
			        NULL,
					NULL,
					OFN_HIDEREADONLY,
					"Bitmap Files (*.bmp)|*.bmp|"
					"All Files (*.*)|*.*||",
					NULL);
	if (dlg.DoModal() != IDOK) return;

	// create a new image
	C3dImage* pImg = new C3dImage;
	if (!pImg->Load(dlg.m_ofn.lpstrFile)) {
		AfxMessageBox("Failed to create image");
		delete pImg;
		return;
	}

	// add the image to the list
	ASSERT(m_pImgList);
	m_pImgList->Append(pImg);

	// add the new image to the combo box
	int i = m_cbImage.AddString(pImg->GetName());

	// make it the current selection
	m_cbImage.SetCurSel(i);
}

void CTexPage::OnApplyTexture() 
{
	UpdateData(TRUE);

	// Get the image from the selected name	in the combo box
	ASSERT(m_pImgList);
	CString strImg;
	m_cbImage.GetWindowText(strImg);
	C3dImage* pImg = m_pImgList->Find(strImg);
	ASSERT(pImg);

	// valiate that the image is suitable
	// I.e. that its sizes are powers of 2
	for (int i = 0; (1 << i) < pImg->GetWidth(); i++);
	for (int j = 0; (1 << j) < pImg->GetHeight(); j++);
	if (pImg->GetWidth() != (1 << i) || pImg->GetHeight() != (1 << j)) {
		AfxMessageBox("This image can't be used as a texture. "
			          "Its sides are not exact powers of 2");
		return;
	}

	// Create a texture from the image
	C3dTexture tex;
	BOOL b;
	b = tex.Create(pImg);
	ASSERT(b);

	// Attach the texture to the shape
	ASSERT(m_pDlg);
	C3dShape* pShape = m_pDlg->GetShape();

	// Create a wrap
	C3dWrap wrap;
	if (IsDlgButtonChecked(_3D_IDC_RADIO6)) {
		b = wrap.Create(RLWrapChrome,
						NULL,
						0, 0, 0,
						0, 0, 1,
						0, 1, 0,
						0, 0,
						1, 1);
	} else if (IsDlgButtonChecked(_3D_IDC_RADIO5)) {
		b = wrap.Create(RLWrapSphere,
						NULL,
						0, 0, 0,
						0, 0, 1,
						0, 1, 0,
						0, 0,
						1, 1);
	} else if (IsDlgButtonChecked(_3D_IDC_RADIO4)) {
		b = wrap.Create(RLWrapCylinder,
						NULL,
						0, 0, 0,
						0, 0, 1,
						0, 1, 0,
						0, 0,
						1, 1);
	} else {
		b = wrap.Create(RLWrapFlat,
						NULL,
						0, 0, 0,
						0, 0, 1,
						0, 1, 0,
						0, 0,
						1, 1);
	}

	if (pShape) {
		if (IsDlgButtonChecked(_3D_IDC_RADIO1)) {
			// apply the texture and wrap to the shape
			pShape->SetTexture(&tex);
			wrap.Apply(pShape);
		} else {
			// apply the texture and wrap to one face
			pShape->SetFaceTexture(m_iFace, &tex);
			wrap.Apply(pShape, m_iFace);
		}
	}
}
